#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# import pulsar
# from pulsar.schema import JsonSchema, Record, String
# from pulsar import ConsumerType
from base.scope import Scope
import socket
import threading
import json
from base.model.enum.topic_type import TopicType

import sys
import os

current_dir = os.path.dirname(os.path.abspath(__file__))
lib_dir = os.path.join(current_dir, 'pulsar_client_libs')
sys.path.insert(0, lib_dir)


import pulsar
from pulsar.schema import JsonSchema, Record, String
from pulsar import ConsumerType


class ListenerMessageDto(Record):
    to = String()
    body = String()
    type = String()
    sender = String()


class PulsarMessageClient(threading.Thread):
    def __init__(self):
        scope = Scope().get_instance()

        self.logger = scope.get_logger()
        self.configuration_manager = scope.get_configuration_manager()
        self.event_manger = scope.get_event_manager()
        self.client = None 
        self.producer = None 
        self.consumer = None 
        self.auth_type = "customAuth"
        self.db_service = scope.get_db_service()

        self.username = str(self.configuration_manager.get('CONNECTION', 'uid'))
        self.password = str(self.configuration_manager.get('CONNECTION', 'password'))
        self.msg = ListenerMessageDto()
        self.consumer_topic_name = "task-{0}".format(self.username)
        

    def connect(self):
        """
        Establishes a connection to the Pulsar server.
        Returns:
            Boolean: If the connection is successful True, otherwise False.
        """
        try:
            self.logger.info("Trying to connect to Pulsar...")
            self.host = str(socket.gethostbyname(self.configuration_manager.get('PULSAR', 'pulsar_host')))
            self.port = str(self.configuration_manager.get('PULSAR', 'pulsar_port'))
            self.tls_trust_certs_file_path = str(self.configuration_manager.get('PULSAR', 'tls_trust_certs_file_path'))
            if self.configuration_manager.get('PULSAR', 'pulsar_use_tls').strip().lower() == 'true':
                self.use_tls = True
                self.pulsar_server_url = "pulsar+ssl://{server_url}:{port}".format(server_url=self.host, port=self.port)
            else:
                self.use_tls = False
                self.pulsar_server_url = "pulsar://{server_url}:{port}".format(server_url=self.host, port=self.port)
            
            self.client = pulsar.Client(
                service_url=self.pulsar_server_url,
                authentication=pulsar.AuthenticationBasic(self.username, self.password, "customAuth"),
                use_tls=self.use_tls,
                tls_trust_certs_file_path=self.tls_trust_certs_file_path,
                tls_allow_insecure_connection=False,
                tls_validate_hostname=False,
                connection_timeout_ms= 60000,
                operation_timeout_seconds=90000
            )
            try:
                test_producer = self.client.create_producer(TopicType.TEST_TOPIC.value)
                test_producer.send(b'Test message')
                self.logger.info("Connected to Pulsar server!")
                test_producer.close()
                return True
            except Exception as e:
                self.logger.error(f"Producer connection failed: {str(e)}")
                return False
        except pulsar.PulsarException as e:  
            self.logger.error(f"Pulsar connection failed: {str(e)}")
            return False
        except Exception as e:
            self.logger.error(f"Failed to connect to Pulsar: {str(e)}")
            return False

    
    #producer method
    def send_direct_message(self, message=None, topic=None):
        producer = None
        if self.client:
            body = json.loads(str(message))
            self.msg = ListenerMessageDto(
                to="lider_sunucu",
                body=message,
                type=topic,
                sender=self.username
            ) 
            try:
                # await asyncio.sleep(1)
                producer = self.client.create_producer(
                    topic=str(topic),
                    schema=JsonSchema(ListenerMessageDto),
                    compression_type=pulsar.CompressionType.ZSTD,
                    block_if_queue_full=True,
                    batching_enabled=False,
                    batching_max_publish_delay_ms=10,
                    properties={
                        "producer-name": self.username,
                        "producer-id": self.username
                })

                def callback(res, msg_id):
                    if res == pulsar.Result.Ok:
                        self.logger.info(f"<<-------- Message published successfully! Message: {body}")
                    else:
                        self.logger.error(f"Failed to publish message: {res}")
                producer.send_async(self.msg, callback)
                producer.flush()
                producer.close()

            except Exception as e:
                self.logger.error('A problem occurred while sending direct message. Error Message: {0}'.format(str(e)))
        else:
            self.logger.error('No connection to pulsar server')


    def recv_direct_message(self):
        if self.client:
            try:
                self.consumer = self.client.subscribe(
                    topic=str(self.consumer_topic_name),
                    subscription_name="ahenk-"+ self.username,
                    consumer_type=ConsumerType.Exclusive, 
                    properties={
                        "consumer-name": self.username,
                        "consumer-id": self.username,
                    })

                while True:
                    try:
                        if self.consumer.is_connected():
                            msg = self.consumer.receive()
                            msg_data = msg.data().decode("utf-8")
                            task_message = json.loads(msg_data)
                            body = task_message.get("body")
                            message_type = json.loads(body)["type"]
                            
                            if message_type == "EXECUTE_TASK":
                                json_body = json.loads(body)
                                task = json_body['task']
                                if isinstance(task, str):
                                    task = json.loads(task)
                                parameter_map = task['parameterMap']
                                use_file_transfer = json_body['fileServerConf']
                                is_password = False
                                for key, value in parameter_map.items():
                                    if "password" in key.lower():
                                        parameter_map[key] = "********"
                                        task['parameterMap'] = parameter_map
                                        json_body['task'] = task
                                        is_password = True
                                if use_file_transfer != None:
                                    file_server_conf = json_body['fileServerConf']
                                    file_server_param = file_server_conf['parameterMap']
                                    for key, value in file_server_param.items():
                                        if "password" in key.lower():
                                            file_server_param[key] = "********"
                                            file_server_conf['parameterMap'] = file_server_param
                                            json_body['fileServerConf'] = file_server_conf
                                    is_password = True
                                if is_password:
                                    self.logger.info('---------->Received message: {0}'.format(str(json_body)))
                                else:
                                    self.logger.info('---------->Received message: {0}'.format(str(task_message.get("body"))))
                            
                            self.event_manger.fireEvent(str(message_type), str(body))
                            self.consumer.acknowledge(msg)
                        else:
                            self.logger.info("No connection with Pulsar server. Reconnecting...")
                            break
                    except pulsar.Interrupted:
                        self.logger.error(f"Error during message reception: {str(e)}")
                        break
            
            except pulsar.PulsarException as e:
                self.logger.info(pulsar.PulsarException)
                self.logger.error('A problem occurred while receiving direct message. Error Message: {0}'.format(str(e)))
                self.consumer.negative_acknowledge(msg)
            
            finally:
                if self.consumer:
                    self.consumer.close()
                if self.client:
                    self.disconnect()
        else:
            self.logger.error('No connection to pulsar server')

    def disconnect(self):
        self.client.close()
        self.logger.info("Disconnected server")
