#!/usr/bin/python3
# -*- coding: utf-8 -*-

from base.plugin.abstract_plugin import AbstractPlugin
import importlib


class ChangeEtaClassroomUnit(AbstractPlugin):
    def __init__(self, task, context):
        super(ChangeEtaClassroomUnit, self).__init__()
        self.task = task
        self.context = context
        self.logger = self.get_logger()
        self.message_code = self.get_message_code()
        self.eta_classroom_unit = self.task['etaClassroomUnit']
        self.logger.debug('Parameters were initialized.')
        
        try:
            self.unit_module = importlib.import_module('etainfo.unit')
        except ImportError:
            self.unit_module = None
            

    def handle_task(self):
        
        try:
            if self.unit_module is not None:
                self.unit_module.set(self.eta_classroom_unit)
                self.logger.info('ETA Classroom Unit name is changed successfully.')
                self.context.create_response(code=self.message_code.TASK_PROCESSED.value,
                                         message='Şube adı başarıyla değiştirildi.')
            else:
                self.logger.error("eta info unit module not found")
                self.context.create_response(code=self.message_code.TASK_ERROR.value,
                                            message='Şube adı değiştirilmedi (etainfo.unit modülü bulunamadı)')

        except Exception as e:
            self.logger.error(
                'A problem occured while handling ETA classroom unit task: {0}'.format(str(e)))
            self.context.create_response(code=self.message_code.TASK_ERROR.value,
                                        message='Şube adı değiştirilirken hata oluştu.')


def handle_task(task, context):
    hostname = ChangeEtaClassroomUnit(task, context)
    hostname.handle_task()
