#!/usr/bin/python3
# -*- coding: utf-8 -*-


from base.model.enum.content_type import ContentType
import json
from base.plugin.abstract_plugin import AbstractPlugin
from base.scope import Scope

class ChangeHostname(AbstractPlugin):
    def __init__(self, data, context):
        super(AbstractPlugin, self).__init__()
        scope = Scope().get_instance()
        self.data = data
        self.context = context
        self.logger = self.get_logger()
        self.message_code = self.get_message_code()
        self.conf_manager = scope.get_configuration_manager()

    def handle_task(self):
        try:
            hostname = self.data['new_hostname']
            with open('/etc/hostname', 'w') as file:
                file.write(hostname + '\n')
        
            with open('/etc/hosts', 'r') as file:
                lines = file.readlines()
            with open('/etc/hosts', 'w') as file:
                for line in lines:
                    if '127.0.1.1' in line:
                        file.write(f'127.0.1.1 {hostname}\n')
                    else:
                        file.write(line)

            self.context.create_response(code=self.message_code.TASK_PROCESSED.value,
                                         message='Bilgisayar adı başarı ile değiştirildi.',
                                         data=json.dumps({'new_hostname': hostname}),
                                         content_type=ContentType.APPLICATION_JSON.value)


        except Exception as e:
            self.logger.error("An error on handle task. Error: " + str(e))
            self.context.create_response(code=self.message_code.TASK_ERROR.value,
                                         message='Bilgisayar adı değiştirilirken hata olustu' + str(e),
                                         content_type=ContentType.APPLICATION_JSON.value)


def handle_task(task, context):
    cls = ChangeHostname(task, context)
    cls.handle_task()
