#!/usr/bin/python
# -*- coding: utf-8 -*-
# Author: Ahmet Karaca <ahmet.karaca@pardus.org.tr> 

from base.plugin.abstract_plugin import AbstractPlugin
import json

class GetParentalControlConfig(AbstractPlugin):
    def __init__(self, data, context):
        super(AbstractPlugin, self).__init__()
        self.data = data
        self.context = context
        self.logger = self.get_logger()
        self.message_code = self.get_message_code()

    def handle_task(self):
        try:
            # file_path = self.data['file-path']
            config = {}
            application_list = {}
            config_path = "/var/lib/eta/eta-kisit/profiles.json"
            application_list_path = "/var/lib/eta/eta-kisit/installed_applications.json"
            is_parental_control_exist = False
            is_application_list_exist = False

            if self.is_exist(config_path):
                self.logger.info("Config exist: " + config_path)
                is_parental_control_exist = True
                config = self.read_file(config_path)

            if self.is_exist(application_list_path):
                self.logger.info("Application List exist: " + application_list_path)
                is_application_list_exist = True
                application_list = self.read_file(application_list_path)

            if not is_parental_control_exist and not is_application_list_exist:
                self.logger.info("Config and Application List not exist")
                self.context.create_response(code=self.message_code.TASK_ERROR.value,
                                             message='ETA Sınırlı Erişim Modu ve Uygulama listesi bulunamadı.',
                                             content_type=self.get_content_type().APPLICATION_JSON.value)
            else:
                self.logger.info("Config: " + json.dumps(config) + " Application List: " + json.dumps(application_list))
                self.context.create_response(code=self.message_code.TASK_PROCESSED.value,
                                             message='ETA Sınırlı Erişim Modu ve Uygulama Listesi içerikleri başarıyla alındı.',
                                             data=json.dumps({'application_list': application_list, 'config': config}),
                                             content_type=self.get_content_type().APPLICATION_JSON.value)

        except Exception as e:
            self.logger.error(str(e))
            self.context.create_response(code=self.message_code.TASK_ERROR.value,
                                         message='ETA Sınırlı Erişim Modu ve Uygulama Listesi içerikleri alınırken hata oluştu: {0}'.format(str(e)))

def handle_task(task, context):
    plugin = GetParentalControlConfig(task, context)
    plugin.handle_task()

