#!/usr/bin/python3
# -*- coding: utf-8 -*-

# Author: Ahmet Karaca <ahmet.karaca@pardus.org.tr> 
# Author: Tuncay Çolak <tuncay.colak@tubitak.gov.tr> 

from base.plugin.abstract_plugin import AbstractPlugin
import json

class ParentalControl(AbstractPlugin):
    def __init__(self, task, context):
        super(ParentalControl, self).__init__()
        self.task = task
        self.context = context
        self.message_code = self.get_message_code()
        self.logger = self.get_logger()
        self.file_path = "/var/lib/eta/eta-kisit/profiles.json"

    def handle_task(self):
        try:
            if self.is_installed("eta-kisit"):
                if self.is_exist(self.file_path):
                    self.update_default_profile()
                else:
                    self.create_file(self.file_path)
                    self.update_default_profile()

                result_code, p_out, p_err = self.execute("eta-kisit --reload")
                self.context.create_response(code=self.message_code.TASK_PROCESSED.value, 
                                            message='ETA Sınırlı Erişim Modu başarıyla uygulandı', data=json.dumps({'Result': 'ok'}), 
                                            content_type=self.get_content_type().APPLICATION_JSON.value)
            else:
                self.logger.debug("eta-kisit application not found")
                self.context.create_response(code=self.message_code.TASK_ERROR.value, 
                                         message='ETA Sınırlı Erişim uygulaması bulunamadı')

        except Exception as e:
            self.logger.error(str(e))
            self.context.create_response(code=self.message_code.TASK_ERROR.value, 
                                         message='ETA Sınırlı Erişim modu ayarlanırken hata oluştu: {0}'.format(str(e)))

    def update_default_profile(self):
        defaultProfile = self.task['defaultProfile']
        current_content = self.read_file(self.file_path)
        current_profile_data = json.loads(current_content)

        default_profile_name = None
        for profile_name, profile_data in current_profile_data["profile_list"].items():
            if "is_default" in profile_data and profile_data["is_default"]:
                default_profile_name = profile_name
                break
        if default_profile_name is None:
            self.logger.debug("Default profile not found. Default profile will be added")
            # raise ValueError("Default profile not found.")
        else:
            del current_profile_data["profile_list"][default_profile_name]

        current_profile_data["profile_list"].update(defaultProfile)
        new_profile_name = next(iter(defaultProfile))
        current_profile_data["current_profile"] = new_profile_name
        self.write_file(self.file_path, json.dumps(current_profile_data, indent=4))

def handle_task(task, context):
    clz = ParentalControl(task, context)
    clz.handle_task()


